﻿using System.Collections.ObjectModel;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using Syncfusion.UI.Xaml.Charts;
using System;
using System.Collections.Generic;
using System.Windows;
using Syncfusion.Windows.SampleLayout;
using Syncfusion.Windows.Shared;

namespace WPF
{
    public partial class MainWindow
    {

        public MainWindow()
        {
            InitializeComponent();
        }
    }

    public class ViewModel
    {
        private readonly Random _rnd = new Random();
        public ChartSeriesCollection Series { get; set; }
        public ICommand BuildPieCommand
        {
            get { return new DelegateCommand<object>(DoSomething); }
        }

        private void DoSomething(object instance)
        {
            BuildPie();
        }

        public ViewModel()
        {
            Series = new ChartSeriesCollection();
        }

        private void BuildPie()
        {
            Series.Clear();
            var n = _rnd.Next(2, 5);
            for (int i = 1; i <= n; i++)
            {
                var series = new PieSeries
                {
                    ItemsSource = BuilModel(),
                    XBindingPath = "Country",
                    YBindingPath = "Count",
                    Label = "series " + i
                };

                Series.Add(series);
                series.Loaded += series_Loaded;
            }
        }
        
        void series_Loaded(object sender, RoutedEventArgs e)
        {
            var z = sender as PieSeries;
            if (z.Area == null) return;
            if (z.Area.Series == null) return;
            if (z.Area.Series.Count == 0) return;
            if (z.Area.Series[0] is PieSeries == false) return;
            if (z.Area.Series == null) return;
            if (z.Area.Series.Count == 0) return;

            UpdateSeriesDefaultColor(z);
            UpdateLegendItems(z);
        }

        private static void UpdateLegendItems(PieSeries z)
        {
            var points = z.Area.Series[0].ItemsSource as List<Model>;
            if (points == null) return;

            var legendItems = new ObservableCollection<LegendItem>();

            for (int i = 0; i < points.Count; i++)
            {
                var item = new LegendItem { Series = z.Area.Series[0], Item = points[i], Label = points[i].Country };
                legendItems.Add(item);

                var binding = new Binding { Source = z.Area.Series[0], Path = new PropertyPath("Interior"), Converter = new InteriorConverter(z.Area.Series[0]), ConverterParameter = i };
                BindingOperations.SetBinding(item, LegendItem.InteriorProperty, binding);
            }
            var chartLegend = z.Area.Legend as ChartLegend;
            if (chartLegend != null) chartLegend.ItemsSource = legendItems;
        }

        private static void UpdateSeriesDefaultColor(PieSeries z)
        {
            z.Area.Series[0].Interior = new SolidColorBrush(Color.FromRgb(255, 0, 0)); //Does not work : It change all item of the first Pie...

            //TODO how can I do it ?
        }

        #region Model

        public List<Model> BuilModel()
        {
            var modelValues = new List<Model>
                    {
                        RamdomModel(),
                        RamdomModel(),
                        RamdomModel(),
                        RamdomModel(),
                        RamdomModel(),
                        RamdomModel() 
                    };
            return modelValues;
        }

        private Model RamdomModel()
        {
            var model = new Model { Count = _rnd.Next(50, 300) };
            switch (_rnd.Next(1, 5))
            {
                case 1:
                    model.Country = "Canada";
                    break;
                case 2:
                    model.Country = "Argentina";
                    break;
                case 3:
                    model.Country = "USA";
                    break;
                case 4:
                    model.Country = "Japan";
                    break;
                case 5:
                    model.Country = "China";
                    break;
            }

            return model;
        }

        #endregion
    }

    public class Model
    {
        public string Country { get; set; }
        public int Count { get; set; }
    }
}
